const blessed = require('blessed');

const prettyjson = require('prettyjson');

const Controller = require('./controller');

let screen = false;

let stateView = false;

let stateInput = false;

let stateInputForm = false;

let currentInput = false;

const inputs = {
  string:{
    get: getStringInput
  },
  bool: {
    get: getBoolInput
  },
  array: {
    get: getArrayInput
  }
}

function init(){

  screen = blessed.screen({
    smartCSR: true
  });

  stateView = blessed.box({
    top: '0%',
    left: '0%',
    width: '100%',
    height: '50%',
    scrollbar: true,
    label: '{bold} State {/bold}',
    tags: true,
    border: {
      type: 'line'
    },
    style: {
      fg: 'white',
      border: {
        fg: 'white'
      },
      focus: {
        border: {
          fg: 'green'
        }
      },
      hover: {
      }
    }
  });

  stateInput = blessed.box({
    top: '50%',
    left: '0%',
    width: '100%',
    height: '50%',
    label: '{bold} Input {/bold}',
    tags: true,
    border: {
      type: 'line'
    },
    content: '\n{center}Loading...{/center}',
    style: {
      fg: 'white',
      border: {
        fg: '#f0f0f0'
      },
      focus: {
        border: {
          fg: 'green'
        }
      },
      hover: {
      }
    }
  });

  stateInputForm = blessed.form({
    parent: stateInput,
    name: 'form',
    bottom: 0,
    left: 0,
    width: '99%',
    height: '50%',
  });

  screen.append(stateView);
  screen.append(stateInput);

  screen.focusPush(stateView);
  screen.focusPush(stateInput);

  screen.key(['tab'], function(ch, key) {
    if(stateView.focused) stateInput.focus();
    else stateView.focus();
    screen.render();
  });

  screen.key(['escape', 'q', 'C-c'], function(ch, key) {
    return process.exit(0);
  });

  screen.render();
}

/**
 * Update state list
 */
function updateState(state){
  stateView.setContent(prettyjson.render(state));
  //stateView.setContent(JSON.stringify(state));
  screen.render();
}

function log(data){
  stateView.setContent(data);
  screen.render();
}

function updateInput(input){

  currentInput = (typeof inputs[input.type] === 'object') ? inputs[input.type].get(input) : inputs.string.get(input);

  stateInput.setContent(`{center}${input.text}{/center}`);

  screen.render();
}

function getStringInput(input){
  const textInput = blessed.textarea({
    parent: stateInputForm,
    name: 'input',
    inputOnFocus: true,
    input: true,
    keys: true,
    bottom: 0,
    left: '1%',
    height: 1,
    width: '98%',
    style: {
      fg: 'white',
      bg: 'black',
      focus: {
        bg: 'red',
        fg: 'white'
      }
    }
  });

  textInput.key('enter', function(ch, key) {
    Controller.get('params').setParam(currentInput.getText().replace('\n', ''));
    stateInputForm.remove(currentInput);
    Controller.get('params').nextOption();
    screen.render();
  });

  textInput.readInput();

  return textInput;
}

function getBoolInput(input){
  const radioset = blessed.radioset({
    parent: stateInputForm,
    name: 'input',
    width: '98%',
    left: '1%',
    height: 5,
    bottom: 0,
    inputOnFocus: true,
    input: true,
    keys: true
  });
  const yes = blessed.radiobutton({
    parent: radioset,
    name: 'like',
    content: 'True',
    left: '40%'
  });
  const no = blessed.radiobutton({
    parent: radioset,
    name: 'like',
    content: 'False',
    left: '55%'
  });

  radioset.focus();

  yes.check();

  radioset.key('y', function(ch, key) {
    yes.check();
    screen.render();
  });

  radioset.key('n', function(ch, key) {
    no.check();
    screen.render();
  });

  radioset.key('enter', function(ch, key) {
    Controller.get('params').setParam(yes.checked);
    stateInputForm.remove(currentInput);
    Controller.get('params').nextOption();
    screen.render();
  });

  return radioset;
}

function getArrayInput(input){
  const button = blessed.button({
    parent: stateInputForm,
    name: 'input',
    bottom: 1,
    height: 3,
    left: '20%',
    width: '60%',
    content: 'Create (Press SPACE)',
    inputOnFocus: true,
    padding: 1,
    input: true,
    align: 'center',
    keys: true,
    mouse: true,
    style: {
      bg: 'white',
      fg: 'black'
    }
  });

  button.focus();

  button.key('space', function(ch, key){
    stateInputForm.remove(currentInput);
    Controller.get('params').inOption();
    screen.render();
  });

  button.key('enter', function(ch, key) {
    stateInputForm.remove(currentInput);
    Controller.get('params').nextOption();
    screen.render();
  });

  return button;
}

function destroy(){
  screen.destroy();
}

module.exports = {
  init,
  updateState,
  updateInput,
  log,
  destroy
}